@@ -7,7 +7,11 @@ from enum import Enum
 import functools
 import os
 import json
+import signal
 import ssl
+
+from uuid import uuid4
+
 import urllib.parse
 import urllib3
 import requests
@@ -29,10 +33,15 @@ def print_openocd_error(error):
     logfile = parsed['logfile']
     if not logfile:
         return
+    error_printed = False
     for line in logfile.splitlines():
         if 'Error: ' in line:
+            error_printed = True
             click.secho(line, fg='red', err=True)
 
+    if not error_printed:
+        click.secho('OpenOCD failed to start', fg='red', err=True)
+
 def print_docker_error(ctx, error):
     """
         Parse an openocd log file and print the error lines
@@ -46,12 +55,28 @@ def print_docker_error(ctx, error):
     click.echo(stderr, err=True, nl=False)
     ctx.exit(parsed['returncode'])
 
-OPENOCD_ERROR_CODES = set((
+def print_canbus_error(ctx, error):
+    if not error:
+        return
+    parsed = json.loads(error)
+    if parsed['stdout']:
+        click.secho(parsed['stdout'], fg='red', nl=False)
+    if parsed['stderr']:
+        click.secho(parsed['stderr'], fg='red', err=True, nl=False)
+        if parsed['stderr'] == 'Cannot find device "can0"\n':
+            click.secho('Please check adapter connection', fg='red', err=True)
+
+
+OPENOCD_ERROR_CODES = {
     'openocd_start_failed',
-))
+}
+
+DOCKER_ERROR_CODES = set()
+
+CANBUS_ERROR_CODES = {
+    'canbus_up_failed',
+}
 
-DOCKER_ERROR_CODES = set((
-))
 
 def quote(gateway):
     return urllib.parse.quote(str(gateway), safe='')
@@ -86,12 +111,12 @@ class LagerSession(BaseUrlSession):
             if error['code'] == 'gateway_timeout_error':
                 raise GatewayTimeoutError(error['description'])
 
-            if error['code'] == 'image_running':
-                click.echo('Script already running - please wait for it to finish or use `lager python --kill` to forcibly terminate it. ', err=True)
-            elif error['code'] in OPENOCD_ERROR_CODES:
+            if error['code'] in OPENOCD_ERROR_CODES:
                 print_openocd_error(error['description'])
             elif error['code'] in DOCKER_ERROR_CODES:
                 print_docker_error(ctx, error['description'])
+            elif error['code'] in CANBUS_ERROR_CODES:
+                print_canbus_error(ctx, error['description'])
             else:
                 click.secho(error['description'], fg='red', err=True)
             ctx.exit(1)
@@ -101,9 +126,11 @@ class LagerSession(BaseUrlSession):
 
         r.raise_for_status()
 
-    def __init__(self, ctx, auth, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._connection_exception = None
+    def __init__(self, auth, *args, response_hook=None, **kwargs):
+        host = os.getenv('LAGER_HOST', _DEFAULT_HOST)
+        base_url = '{}{}'.format(host, '/api/v1/')
+
+        super().__init__(*args, base_url=base_url, **kwargs)
         verify = 'NOVERIFY' not in os.environ
         if not verify:
             urllib3.disable_warnings()
@@ -113,7 +140,10 @@ class LagerSession(BaseUrlSession):
                 'Authorization': '{} {}'.format(auth['type'], auth['token'])
             }
             self.headers.update(auth_header)
-        self.headers.update({'Lager-Version': __version__})
+        self.headers.update({
+            'Lager-Version': __version__,
+            'Lager-Invocation-Id': str(uuid4()),
+            })
         ci_env = get_ci_environment()
         if ci_env == CIEnvironment.HOST:
             self.headers.update({'Lager-CI-Active': 'False'})
@@ -122,13 +152,19 @@ class LagerSession(BaseUrlSession):
             self.headers.update({'Lager-CI-System': ci_env.name})
 
         self.verify = verify
-        self.hooks['response'].append(functools.partial(LagerSession.handle_errors, ctx))
+        if response_hook:
+            self.hooks['response'].append(response_hook)
 
 
     def request(self, *args, **kwargs):
         """
             Catch connection errors so they can be handled more cleanly
         """
+
+        if 'headers' not in kwargs:
+            kwargs['headers'] = {}
+        kwargs['headers'].update({'Lager-Request-Id': str(uuid4())})
+
         try:
             return super().request(*args, **kwargs)
         except requests.exceptions.ConnectTimeout:
@@ -173,12 +209,12 @@ class LagerSession(BaseUrlSession):
         url = 'gateway/{}/run-python'.format(quote(gateway))
         return self.post(url, files=files, stream=True)
 
-    def kill_python(self, gateway, sig):
+    def kill_python(self, gateway, sig=signal.SIGTERM):
         """
             Run python on a gateway
         """
         url = 'gateway/{}/kill-python'.format(quote(gateway))
-        return self.post(url, json={'signal': sig}, stream=True)
+        return self.post(url, json={'signal': sig})
 
     def gateway_hello(self, gateway):
         """
@@ -334,16 +370,36 @@ class LagerSession(BaseUrlSession):
         url = 'gateway/{}/wifi/delete-connection'.format(quote(gateway))
         return self.post(url, json={'ssid': ssid})
 
+    def can_up(self, gateway, bitrate):
+        """
+            Bring up the CAN bus
+        """
+        url = 'gateway/{}/canbus/up'.format(quote(gateway))
+        return self.post(url, json={'bitrate': bitrate})
+
+    def can_send(self, gateway, frames):
+        """
+            Send one or more frames on CAN bus
+        """
+        url = 'gateway/{}/canbus/send'.format(quote(gateway))
+        return self.post(url, json={'frames': [frame._asdict() for frame in frames]})
+
+    def can_dump(self, gateway, can_options):
+        """
+            Dump frames from CAN bus
+        """
+        url = 'gateway/{}/canbus/dump'.format(quote(gateway))
+        return self.post(url, json={'can_options': can_options})
+
+
 class LagerContext:  # pylint: disable=too-few-public-methods
     """
         Lager Context manager
     """
     def __init__(self, ctx, auth, defaults, debug, style):
-        host = os.getenv('LAGER_HOST', _DEFAULT_HOST)
         ws_host = os.getenv('LAGER_WS_HOST', _DEFAULT_WEBSOCKET_HOST)
-        base_url = '{}{}'.format(host, '/api/v1/')
-
-        self.session = LagerSession(ctx, auth, base_url=base_url)
+        response_hook = functools.partial(LagerSession.handle_errors, ctx)
+        self.session = LagerSession(auth, response_hook=response_hook)
         self.defaults = defaults
         self.style = style
         self.ws_host = ws_host
